import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.util.List;

/**
 * A subclass of SmoothMover, which represents a shot and inflicts damage when it crosses a player or an object.
 * 
 * @author Romuald Apothélos
 * @author Boris Méan 
 * 
 * @version 14.01.2022
 */
public class Bullet extends SmoothMover
{
    public Actor killer;
    public int damage;
    public boolean gravity =false;
    /**
     * constructor of the class bullet
     */
    public Bullet(int rotation,Actor player, int dg,int special)
    {
        addForce(new Vector(rotation,10.0));
        turn(rotation);
        killer= player;
        damage = dg;
        GreenfootSound sound= new GreenfootSound("bullet.mp3");
        sound.play();
        sound.setVolume(40);
        if(special==1)
        {
         setImage(new GreenfootImage("arrow.png"));
         gravity=true;
        }
        else if(special==2)
        {
         setImage(new GreenfootImage("b.pass.png"));   
        }
    }
    /**
     * Act - do whatever the bullet wants to do. This method is called whenever
     * the 'Act' or 'Run' button gets pressed in the environment.
     */
    public void act()
    {
        if(collision()){return;};
        if(gravity){gravity(this.getMovement(),false);}
        move();
    }
    /**
     * collision - ceck if there is object to give damage to and disappear
     */
    public boolean collision()
    {
        if( getY()<=5 || getY()>=595 || getX()<=5 || getX()>=1195 )
        {
            getWorld().removeObject(this);
            return true;
        }
        if(getOneIntersectingObject(Player.class) != null && getOneIntersectingObject(Player.class) != killer)
        {
           List<Player> player = (List<Player>) getWorld().getObjects(Player.class);
           for (Player play : player) 
            {
            if (play == getOneIntersectingObject(Player.class))
            {
                play.hurt(damage,killer);
                getWorld().removeObject(this);
                return true;
            }
            }
           return true;
        }
        else if(getOneIntersectingObject(Obstacle.class) != null )
        {
           List<Obstacle> obstacle = (List<Obstacle>) getWorld().getObjects(Obstacle.class);
           for (Obstacle obs : obstacle) 
            {
            if (obs == getOneIntersectingObject(Obstacle.class))
            {
                obs.destroy(damage,killer);
                getWorld().removeObject(this);
                return true;
            }
            }
           return true;
        }
        else if(getOneIntersectingObject(Flag.class) != null )
        {
           List<Flag> flag = (List<Flag>) getWorld().getObjects(Flag.class);
           for (Flag fla : flag) 
            {
            if (fla == getOneIntersectingObject(Flag.class))
            {
                fla.hurt(damage,killer);
                getWorld().removeObject(this);
                return true;
            }
            }
           return true;
        }
        return false;
    }
}
