import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.util.List;
import greenfoot.Actor;

/**
 * Class of object that stays temporarily in one place and can affect the players
 * 
 * @author Romuald Apothélos
 * @author Boris Méan 
 * 
 * @version 8.12.2021
 */
public class GroundSticker extends SmoothMover
{
    public int type;
    public boolean ground = false;
    public int time = 0;
    public Actor thrower;
    /**
     * contructor of the class GroundSticker
     */
    public GroundSticker(int tp, Actor player)
    {
        type = tp;
        thrower = player;
        if(tp==0)
        {
            setImage(new GreenfootImage("fire."+(Greenfoot.getRandomNumber(2)+1)+".png"));
            GreenfootSound sound= new GreenfootSound("fire.mp3");
            sound.play();
            sound.setVolume(50);
        }
        else
        {
           setImage(new GreenfootImage("fromage.png")); 
        }
    }
    /**
     * Act - do whatever the thrower wants to do. This method is called whenever
     * the 'Act' or 'Run' button gets pressed in the environment.
     */
    public void act()
    {
        if(type==0)
        {
            burn();
        }
        if(type==1)
        {
            stick();
        }
        Vector movement = getMovement();
        ground=false;
        if(onGround(movement, 5)){ground=true;}
        gravity(movement, ground);
        friction(movement);
        wallchek(movement);
        move();
        time++;
        if(time>600){getWorld().removeObject(this);return;}
    }
    /**
     * Friction - apply friction to the horizontal movement
     */
    public void friction(Vector movement)
    {
        double friction = 0.02;
        if(movement.getX()>0.05)
        {
                addForce (new Vector (-friction, 0.0));
        }
        if(movement.getX()<-0.05)
        {
                addForce (new Vector (friction, 0.0));
        }
        if(ground == true)
        {
                addForce (new Vector (-movement.getX(), 0.0));
        }
    }
    /**
     * Wallchek - check if the object is near a wall and stop him form moving any further
     */
    public void wallchek(Vector movement)
    {
        if(onWall(movement,10)==1 && movement.getX()<=0){
            addForce(new Vector(-movement.getX(),0));
        }else if(onWall(movement,10)==2 && movement.getX()>=0){
            addForce(new Vector(-movement.getX(),0));
        }
    }
    /**
     * Burn - change the image of the fire and affect player.
     */
    public void burn()
    {
        setImage(new GreenfootImage("fire."+(Greenfoot.getRandomNumber(2)+1)+".png"));
        if(getOneIntersectingObject(Player.class) != null)
        {
           List<Player> player = (List<Player>) getWorld().getObjects(Player.class);
           for (Player play : player) 
            {
            if (play == getOneIntersectingObject(Player.class))
            {
                play.hurt(1,thrower);
            }
            }
        }
        else if(getOneIntersectingObject(Obstacle.class) != null )
        {
           List<Obstacle> obstacle = (List<Obstacle>) getWorld().getObjects(Obstacle.class);
           for (Obstacle obs : obstacle) 
            {
            if (obs == getOneIntersectingObject(Obstacle.class))
            {
                obs.destroy(1,thrower);
            }
            }
        }
        else if (getOneIntersectingObject(GroundSticker.class) !=null)
        {
            getWorld().removeObject(getOneIntersectingObject(GroundSticker.class));
        }
    }
    /**
     * Stick - bolque the player
     */
    public void stick()
    {
       if(getOneIntersectingObject(Player.class) != null)
        {
           List<Player> player = (List<Player>) getWorld().getObjects(Player.class);
           for (Player play : player) 
            {
            if (play == getOneIntersectingObject(Player.class))
            {
                Vector movement = play.getMovement();
                play.addForce(new Vector(-movement.getX(),-movement.getY()));
            }
            }
        } 
    }
}
